<?php

/**
 * The plugin bootstrap file
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 * Plugin Name: AI Harness
 * Plugin URI: https://aiharness.io/ai-harness-for-wordpress/
 * Description: ChatGPT, Content Writer, Auto Content Writer, ChatBot, Image Generator, Audio Converter, WooCommerce Product Writer, SEO optimizer, AI Training, Embeddings, Title Suggestions and hundreds of ready to use prompts and forms.
 * Version: 2.0
 * Author: Vibhuti Technology 
 * Author URI: https://vibhuti.biz/
 **/

session_start();
define('PLUGIN_DIR', plugin_dir_path(__FILE__));

require_once __DIR__ . '/variable/global.php';
require_once __DIR__ . '/includes/wp-admin-menu.php';
global $iconsDataArray;
global $promptBaseArray;
global $apiUrl;


function aiharness_activate()
{
    global $table_prefix, $wpdb;
    include_once(PLUGIN_DIR . 'includes/wp-setting-table.php');
    include_once(PLUGIN_DIR . 'includes/wp-account-table.php');
    include_once(PLUGIN_DIR . 'includes/wp-content-table.php');
    include_once(PLUGIN_DIR . 'includes/wp-language-table.php');
    include_once(PLUGIN_DIR . 'includes/wp-user-account-data.php');
    include_once(PLUGIN_DIR . 'includes/wp-audio-converter-table.php');
}

register_activation_hook(__FILE__, 'aiharness_activate');


function aih_aiharness_deactive()
{
}
register_deactivation_hook(__FILE__, "aih_aiharness_deactive");


function aih_aiharness_enqueue_stylesheet()
{
    $plugin_data = get_plugin_data(__FILE__);
    $plugin_version = $plugin_data['Version'];

    wp_enqueue_style('style', plugins_url('assets/css/style.css', __FILE__), array(), $plugin_version);
    wp_enqueue_style('style-account', plugins_url('assets/css/style-account.css', __FILE__), array(), $plugin_version);
}

add_action('admin_print_styles', 'aih_aiharness_enqueue_stylesheet');

// Act on plugin activation
add_action('wp_ajax_aih_account', 'aih_account');

function aih_account()
{
    // Verify nonce
    if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'aih_account_nonce')) {
        wp_send_json_error('Nonce verification failed for aih_account.');
        exit;
    }

    global $wpdb, $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $personal_key = sanitize_text_field($_POST['personal_key']);
    $secert_key =  sanitize_text_field($_POST['secert_key']);
    $api_user_id =  sanitize_text_field($_POST['api_user_id']);
    $plugin_name = "AI Harness";
    $plan = "Free";

    $tablename = $wpdb->prefix . 'account';
    $existing_data = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tablename WHERE api_user_id = %d", $api_user_id));

    if ($existing_data) {
        // Update the existing row
        $wpdb->update(
            $tablename,
            array(
                'publisher_key' => $personal_key,
                'secert_key' => $secert_key,
                'plugin_name' => $plugin_name,
                'plan' => $plan
            ),
            array('api_user_id' => $api_user_id)
        );
        $id = $existing_data->id; // Use existing ID for response
    } else {
        // Insert a new row
        $wpdb->insert(
            $tablename,
            array(
                'user_id' => $user_id,
                'api_user_id' => $api_user_id,
                'publisher_key' => $personal_key,
                'secert_key' => $secert_key,
                'plugin_name' => $plugin_name,
                'plan' => $plan
            )
        );
        $id = $wpdb->insert_id; // Use newly inserted ID for response
    }


    if ($id) {
        $message = "User Account Created Successfully";
        $response = [
            'result'  => 'Success',
            'message' => $message,
        ];

        http_response_code(200);
        echo wp_json_encode($response);
        exit;
    } else {
        http_response_code(400);
        $message = "Something went wrong. Please try again later!";
        echo wp_json_encode(['result' => 'Failure', 'message' => esc_html($message)], JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT);
        exit;
    }
}

// Save Setting Data
add_action('wp_ajax_aih_settng_save', 'aih_settng_save');

function aih_settng_save()
{
    // Verify nonce
    if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'aih_setting_nonce')) {
        wp_send_json_error('Nonce verification failed for aih_settng_save.');
        exit;
    }
    global $wpdb, $api_user_id, $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $model = sanitize_text_field($_POST['model']);
    $temperature = intval($_POST['temperature']);
    $max_token = intval($_POST['max_token']);
    $top_p = intval($_POST['top_p']);
    $best_of = intval($_POST['best_of']);
    $frequency_penalty = intval($_POST['frequency_penalty']);
    $presence_penalty = intval($_POST['presence_penalty']);
    $secretKey = sanitize_text_field($_POST['secretKey']);

    $tablename = $wpdb->prefix . 'setting';
    if ($api_user_id > 0) {
        $existing_data = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tablename WHERE api_user_id = %d", $api_user_id));
        $where_condition = array('api_user_id' => $api_user_id);
    } else {
        $existing_data = $wpdb->get_row($wpdb->prepare("SELECT * FROM $tablename WHERE user_id = %d", $user_id));
        $where_condition = array('user_id' => $user_id);
    }

    if ($existing_data) {
        // Update existing data
        $wpdb->update(
            $tablename,
            array(
                'model' => $model,
                'temperature' => $temperature,
                'max_token' => $max_token,
                'top_p' => $top_p,
                'best_of' => $best_of,
                'frequency_penalty' => $frequency_penalty,
                'presence_penalty' => $presence_penalty
            ),
            $where_condition,
            array('%s', '%d', '%d', '%s', '%d', '%d', '%d', '%d'),
            array('%d')
        );
    } else {
        // Insert new data
        $wpdb->insert(
            $tablename,
            array(
                'user_id' => $user_id,
                'api_user_id' => ($api_user_id != '') ? $api_user_id : 0,
                'model' => $model,
                'temperature' => $temperature,
                'max_token' => $max_token,
                'top_p' => $top_p,
                'best_of' => $best_of,
                'frequency_penalty' => $frequency_penalty,
                'presence_penalty' => $presence_penalty
            ),
            array('%d', '%s', '%f', '%d', '%s', '%f', '%d', '%d', '%d')
        );
    }

    // Send response back to the frontend
    $response = array('result' => 'Success');
    echo wp_json_encode($response);
    wp_die(); // This is required to terminate immediately and return a proper response
}

add_action('wp_ajax_aih_contact_email', 'aih_contact_email');

function aih_contact_email()
{
    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_name = isset($_POST['user_name']) ? sanitize_text_field($_POST['user_name']) : '';
    $user_email = isset($_POST['user_email']) ? sanitize_email($_POST['user_email']) : '';
    $subject = isset($_POST['subject']) ? sanitize_text_field($_POST['subject']) : '';
    $message_area = isset($_POST['message_area']) ? wp_kses_post($_POST['message_area']) : '';

    $mailResult = false;
    $headers = "From: My Name $user_email" . "\r\n";
    $mailResult = wp_mail('admin@aiharness.io', $subject, $message_area, $headers);
    if ($mailResult) {
        $message = "Mail send sccesfully.";
        $response = [
            'result'  => 'Success',
            'message' => $message,
        ];

        http_response_code(200);
        echo wp_json_encode($response);
        exit;
    } else {
        http_response_code(400);
        $message = $message ? $message : "Something went wrong. Please try again later!";
        // echo wp_json_encode(['result' => 'Failure', 'message' => esc_html($message)]);
        echo wp_json_encode(['result' => 'Failure', 'message' => esc_html($message)], JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT);

        exit;
    }
}


add_action('wp_ajax_aih_content_post_page', 'aih_content_post_page');

function aih_content_post_page()
{
     // Verify nonce
     if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'aih_content_post_page_nonce' ) ) {
        wp_send_json_error( array( 'result' => 'Failure', 'message' => 'Nonce verification failed' ) );
        exit;
    }
    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $post_page_title = sanitize_text_field($_POST['post_page_title']);
    $post_page_content = sanitize_text_field($_POST['post_page_content']);
    $attrValue = sanitize_text_field($_POST['attrValue']);

    $wordpress_post = array(
        'post_title' => $post_page_title,
        'post_content' => $post_page_content,
        'post_status' => 'publish',
        'post_author' =>  $user_id,
        'post_type' => $attrValue
    );

    wp_insert_post($wordpress_post);
    $message = "Post Publish Succesfully.";
    $response = [
        'result'  => 'Success',
        'message' => $message,
    ];

    http_response_code(200);
    // echo wp_json_encode($response);
    echo wp_json_encode($response, JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT);


    exit;
}

add_action('wp_ajax_aih_content_data', 'aih_content_data');
function aih_content_data()
{
       // Verify nonce
       if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'aih_content_nonce' ) ) {
        wp_send_json_error( 'Nonce verification failed for aih_content_data.' );
        exit;
    }
    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $title = isset($_POST['title']) ? sanitize_text_field($_POST['title']) : '';
    $model = isset($_POST['model']) ? sanitize_text_field($_POST['model']) : '';
    $token = isset($_POST['token']) ? sanitize_text_field($_POST['token']) : '';
    $type = isset($_POST['type']) ? sanitize_text_field($_POST['type']) : '';

    $author =  $current_user->user_login;
    $date = gmdate('Y-m-d');

    $tablename = $wpdb->prefix . 'content';
    $sql = $wpdb->prepare("INSERT INTO " . $tablename . " (user_id,title,date,token,model,author,type) VALUES ( %d,%s,%s,%s,%s,%s,%s)", $user_id, $title, $date, $token, $model, $author, $type);
    $wpdb->query($sql);
    $id = $wpdb->insert_id;

    if ($id) {
        $message = "Content Inserted Succesfully.";
        $response = [
            'result'  => 'Success',
            'message' => $message,
        ];

        http_response_code(200);
        echo wp_json_encode($response, JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT);


        exit;
    } else {
        http_response_code(400);
        $message = $message ? $message : "Something went wrong. Please try again later!";
        // echo wp_json_encode(['result' => 'Failure', 'message' => $message]);
        echo wp_json_encode(['result' => 'Failure', 'message' => esc_html($message)], JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS | JSON_HEX_QUOT);
        exit;
    }
}

add_action('wp_ajax_aih_publish_content', 'aih_publish_content');
function aih_publish_content()
{
      // Verify nonce
      if ( ! isset( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'aih_content_nonce' ) ) {
        wp_send_json_error( 'Nonce verification failed for aih_content_data.' );
        exit;
    }
    // Retrieve the form field values
    $field1 = isset($_POST['field1']) ? sanitize_text_field($_POST['field1']) : '';
    $tags = isset($_POST['tags']) ? sanitize_text_field($_POST['tags']) : '';
    $category = isset($_POST['category']) ? sanitize_text_field($_POST['category']) : '';
    $admin = isset($_POST['admin']) ? sanitize_text_field($_POST['admin']) : '';


    // Create a new post
    $new_post = array(
        'post_title' => $field1,
        'post_content' => '',
        'post_status' => 'publish',
        'post_author' => $admin,
        'post_category' => array($category),
        'tags_input' => $tags,
    );

    // Insert the new post
    $post_id = wp_insert_post($new_post);

    // Check if the post was successfully published
    if ($post_id) {
        echo "Post successfully published!";
    } else {
        echo "Something went wrong, try again.";
    }
}


add_action('wp_ajax_store_content_data', 'store_content_data');
function store_content_data()
{
    
    $new = array(
        'post_title' => sanitize_text_field($_POST['title']),
        'post_content' => sanitize_textarea_field($_POST['content_text']),
        'post_status' => 'draft'
    );

    $post_id = wp_insert_post($new);

    if ($post_id) {
        // Save the featured image
        if (isset($_POST['featured_generate_image']) && !empty($_POST['featured_generate_image'])) {
            $image_url = esc_url_raw($_POST['featured_generate_image']);

            // Create an attachment from the image URL
            $attachment_id = aih_store_image_as_attachment($image_url);

            // Set the attachment as the post's featured image
            if ($attachment_id) {
                set_post_thumbnail($post_id, $attachment_id);
            }
        }

        // Save the content image
        if (isset($_POST['generate_image']) && !empty($_POST['generate_image'])) {
            $content_image_url = esc_url_raw($_POST['generate_image']);

            // Create an attachment from the content image URL
            $content_attachment_id = aih_store_image_as_attachment($content_image_url);

            // Add the content image to the post content
            if ($content_attachment_id) {
                $image_html = wp_get_attachment_image($content_attachment_id, 'full');
                $post_content = get_post_field('post_content', $post_id);
                $updated_content = $image_html . $post_content;
                wp_update_post(array(
                    'ID' => $post_id,
                    'post_content' => $updated_content,
                ));
            }
        }

        // Send the success response as JSON
        wp_send_json(array('message' => esc_html__('Post Draft successfully', 'text-domain')));
    } else {
        // Send the error response as JSON
        wp_send_json(array('message' => esc_html__('Something went wrong, try again.', 'your-text-domain')));
    }

    exit;
}

function aih_store_image_as_attachment($image_url)
{
    $upload_dir = wp_upload_dir(); // Get the upload directory
    $image_data = file_get_contents($image_url); // Get the image data

    // Get the filename from the image URL
    $file_name = basename($image_url);

    // Generate a unique name for the image
    $random_name = wp_unique_filename($upload_dir['path'], $file_name);

    // Set the file path to the uploads directory
    $file_path = $upload_dir['path'] . '/' . $random_name;

    // Save the image to the uploads directory
    file_put_contents($file_path, $image_data);

    // Prepare the attachment data
    $attachment = array(
        'post_mime_type' => mime_content_type($file_path), // Automatically get the MIME type
        'post_title' => sanitize_file_name($random_name),
        'post_content' => '',
        'post_status' => 'inherit'
    );

    // Insert the attachment into the database
    $attachment_id = wp_insert_attachment($attachment, $file_path);

    // Generate metadata for the attachment (e.g., image sizes)
    require_once ABSPATH . 'wp-admin/includes/image.php';
    $attachment_data = wp_generate_attachment_metadata($attachment_id, $file_path);
    wp_update_attachment_metadata($attachment_id, $attachment_data);

    return $attachment_id; // Return the attachment ID
}


add_action('wp_ajax_aih_content_data_get', 'aih_content_data_get');

function aih_content_data_get()
{
    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $type = sanitize_text_field($_POST['type']);
    $search_query = sanitize_text_field($_POST['search_query']);
    $start = intval($_POST['start']);
    $length = intval($_POST['length']);

    $tablename = $wpdb->prefix . 'content';

    // Modify the SQL query to include the search condition if a search query is provided
    $sql_query = "SELECT `id`,`title`,`date`,`token`,`model`,`author` FROM $tablename WHERE (user_id=$user_id AND type='$type')";

    if (!empty($search_query)) {
        $sql_query .= " AND (title LIKE '%$search_query%' OR token LIKE '%$search_query%' OR model LIKE '%$search_query%' OR author LIKE '%$search_query%')";
    }

    // Count total records with the search condition
    $total_records_query = $sql_query;
    $total_records = $wpdb->get_var("SELECT COUNT(*) FROM ($total_records_query) as total");

    // Add LIMIT clause for pagination
    $sql_query .= " LIMIT $start, $length";
    $userData = $wpdb->get_results($sql_query);

    if ($userData) {
        $message = "Content get Successfully.";
        $response = [
            'result' => 'Success',
            'message' => $message,
            'data' => $userData,
            'recordsTotal' => $total_records,
            'recordsFiltered' => $total_records,
        ];

        http_response_code(200);
        echo wp_json_encode($response);
        exit;
    } else {
        http_response_code(200);
        $message = "No Data Found!!";
        wp_send_json([
            'result' => 'Failure',
            'message' => esc_html($message),
            'recordsTotal' => 0,
            'recordsFiltered' => 0,
            'data' => []
        ]);
        exit;
    }
}

add_action('wp_ajax_aih_language_content', 'aih_language_content');

function aih_language_content()
{
    // Verify nonce
    if (!isset($_POST['aih_language_nonce']) || !wp_verify_nonce($_POST['aih_language_nonce'], 'aih_language_nonce')) {
        wp_send_json_error('Nonce verification failed.');
        exit;
    }

    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $language = sanitize_text_field($_POST['language']);
    $text = sanitize_text_field($_POST['text']);
    $translation = sanitize_text_field($_POST['translation']);
    $token = sanitize_text_field($_POST['token']);
    $model = sanitize_text_field($_POST['model']);
    $author =  $current_user->user_login;
    $date = gmdate('Y-m-d');

    $tablename = $wpdb->prefix . 'language';
    $sql = $wpdb->prepare("INSERT INTO " . $tablename . " (user_id,text,language,translation,date,token,model,author) VALUES ( %d,%s,%s,%s,%s,%s,%s,%s)", $user_id, $text, $language, $translation, $date, $token, $model, $author);
    $wpdb->query($sql);
    $id = $wpdb->insert_id;

    if ($id) {
        $message = "language Log Saved Succesfully.";
        $response = [
            'result'  => 'Success',
            'message' => $message,
        ];

        http_response_code(200);
        echo wp_json_encode($response);
        exit;
    } else {
        http_response_code(400);
        $message = $message ? $message : "Something went wrong. Please try again later!";
        wp_send_json(['result' => 'Failure', 'message' => sanitize_text_field($message)]);

        exit;
    }
}


add_action('wp_ajax_aih_language_data_get', 'aih_language_data_get');

function aih_language_data_get()
{
    $user_id = $current_user->ID;
    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $tablename = $wpdb->prefix . 'language';
    $userData = $wpdb->get_results("SELECT `id`,`language`,`text`,`translation`,`token`,`model`,`author`,`date` FROM $tablename where user_id=$user_id");

    if ($userData) {
        $message = "language Data get successfully.";
        $response = [
            'result'  => 'Success',
            'message' => $message,
            'data' => $userData
        ];
        http_response_code(200);
        echo wp_json_encode($response);
        exit;
    } else {
        http_response_code(400);
        $message = $message ? $message : "Something went wrong. Please try again later!";

        wp_send_json(['result' => 'Failure', 'message' => sanitize_text_field($message)]);

        exit;
    }
}


// Audio To Text Converter 
add_action('wp_ajax_aih_generated_post', 'aih_generated_post');
add_action('wp_ajax_nopriv_aih_generated_post', 'aih_generated_post');

function aih_generated_post()
{
    // Verify nonce
    $nonce = $_POST['aih_generated_post_nonce'];
    if (!wp_verify_nonce($nonce, 'aih_generated_post_nonce')) {
        wp_send_json_error('Nonce verification failed.');
        exit;
    }
    // Get the post data sent from the AJAX request
    $post_data = isset($_POST['post_data']) ? json_decode(stripslashes($_POST['post_data']), true) : '';

    if (empty($post_data)) {
        wp_send_json(['result' => 'failure', 'message' => 'Invalid post data']);
    }

    // Extract the post data
    $title = sanitize_text_field($post_data['title']);
    $content = sanitize_text_field($post_data['content']);
    $category_name = sanitize_text_field($post_data['category']);
    $author_name = sanitize_text_field($post_data['author']);
    $status = sanitize_text_field($post_data['status']);
    $format = sanitize_text_field($post_data['format']);

    // Get the category ID based on the name
    $category_id = 0;
    if ($category_name) {
        $category_object = get_term_by('name', $category_name, 'category');
        if ($category_object) {
            $category_id = $category_object->term_id;
        }
    }

    // Get the author ID based on the name
    $author_id = 0;
    if ($author_name) {
        $user = get_user_by('login', $author_name);
        if ($user) {
            $author_id = $user->ID;
        }
    }

    // Prepare the post data
    $post_data = [
        'post_title' => $title,
        'post_content' => $content,
        'post_status' => $status,
        'post_category' => [$category_id], // Use an array for category IDs
        'post_author' => $author_id
    ];

    // Insert the post into the database
    $post_id = wp_insert_post($post_data);

    if ($post_id) {
        // Set the format as a post meta if needed
        if ($format) {
            update_post_meta($post_id, 'format', $format);
        }
        // Check the post status and send the appropriate success message
        $success_message = ($status === 'draft') ? 'Post draft successfully' : 'Post published successfully';
        wp_send_json(['result' => 'success', 'message' => $success_message, 'post_id' => $post_id]);
    } else {
        wp_send_json(['result' => 'failure', 'message' => 'Failed to save the post. Please try again later.']);
    }
}


add_action('wp_ajax_aih_audio_post_page', 'aih_audio_post_page');
function aih_audio_post_page()
{
    // Verify nonce
    if (!isset($_POST['aih_audio_post_page_nonce']) || !wp_verify_nonce($_POST['aih_audio_post_page_nonce'], 'aih_audio_post_page_nonce')) {
        wp_send_json_error('Nonce verification failed.');
        exit;
    }

    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $post_page_title = isset($_POST['post_page_title']) ? sanitize_text_field($_POST['post_page_title']) : '';
    $post_page_content = isset($_POST['post_page_content']) ? wp_kses_post($_POST['post_page_content']) : '';
    $attrValue = isset($_POST['attrValue']) ? sanitize_text_field($_POST['attrValue']) : '';
    $category = isset($_POST['category']) ? sanitize_text_field($_POST['category']) : '';
    $author = isset($_POST['author']) ? sanitize_text_field($_POST['author']) : '';
    $status = isset($_POST['status']) ? sanitize_text_field($_POST['status']) : '';


    $wordpress_post = array(
        'post_title' => $post_page_title,
        'post_content' => $post_page_content,
        'post_status' => 'publish',
        'post_author' =>  $author,
        'post_type' => $attrValue,
        'post_category' => $category,
        'status' => $status

    );

    wp_insert_post($wordpress_post);
    $message = "Post Publish Succesfully.";
    $response = [
        'result'  => 'Success',
        'message' => $message,
    ];

    http_response_code(200);
    wp_send_json($response);

    exit;
}


add_action('wp_ajax_aih_log_audio_data', 'aih_log_audio_data');

function aih_log_audio_data()
{
    // Verify nonce
    if (!isset($_POST['aih_audio_post_page_nonce']) || !wp_verify_nonce($_POST['aih_audio_post_page_nonce'], 'aih_audio_post_page_nonce')) {
        wp_send_json_error('Nonce verification failed.');
        exit;
    }
    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $title = isset($_POST['title']) ? sanitize_text_field($_POST['title']) : '';
    $purpose = isset($_POST['purpose']) ? sanitize_text_field($_POST['purpose']) : '';
    $category = isset($_POST['category']) ? sanitize_text_field($_POST['category']) : '';
    $author = isset($_POST['author']) ? sanitize_text_field($_POST['author']) : '';
    $status = isset($_POST['status']) ? sanitize_text_field($_POST['status']) : '';

    $date = gmdate('Y-m-d');

    $tablename = $wpdb->prefix . 'audio_converter';
    $sql = $wpdb->prepare("INSERT INTO " . $tablename . " (user_id,title,date,category,author,status,purpose) VALUES ( %d,%s,%s,%s,%s,%s,%s)", $user_id, $title, $date, $category, $author, $status, $purpose);
    $wpdb->query($sql);
    $id = $wpdb->insert_id;

    if ($id) {
        $message = "Log Saved Succesfully.";
        $response = [
            'result'  => 'Success',
            'message' => $message,
        ];

        http_response_code(200);
        echo wp_json_encode($response);
        exit;
    } else {
        http_response_code(400);
        $message = $message ? $message : "Something went wrong. Please try again later!";
        echo wp_json_encode(['result' => 'Failure', 'message' => $message]);
        exit;
    }
}

add_action('wp_ajax_aih_audio_log_data_get', 'aih_audio_log_data_get');
function aih_audio_log_data_get()
{

    $user_id = $current_user->ID;
    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;
    $tablename = $wpdb->prefix . 'audio_converter';
    $userData = $wpdb->get_results("SELECT `id`,`title`,`purpose`,`date`,`category`,`author`,`status` FROM $tablename where user_id=$user_id");

    if ($userData) {
        $message = "language Data get successfully.";
        $response = [
            'result'  => 'Success',
            'message' => $message,
            'data' => $userData
        ];
        http_response_code(200);
        echo wp_json_encode($response);
        exit;
    } else {
        http_response_code(400);
        $message = $message ? $message : "Something went wrong. Please try again later!";
        // echo wp_json_encode(['result' => 'Failure', 'message' => $message]);
        wp_send_json_error($message, 400);

        exit;
    }
}

add_action('wp_ajax_aih_prompt_content', 'aih_prompt_content');
function aih_prompt_content()
{
    include(PLUGIN_DIR . 'includes/wp-global.php');
    $responseData = '';
    $promptBaseIncrement = 1;
    $promptBaseArray  = isset($_POST['promptData']) ? $_POST['promptData'] : array();;
    $responseData .= '<div class="panel-right-side-prompt">';
    $responseData .= '<div class="prompt-ai-wrap"><input type="text" class="text-generator-search" id="search"><button id="search-prompt" onclick="searchPrompt()">Search</button></div>';
    $responseData .= '<div class="panel-right-side-prompt-container">';
    foreach ($promptBaseArray as $key => $promptBaseData) {
        $responseData .= "<a href='javascript:void(0)' class='promptBaseId' attrId='$promptBaseData[id]'>";
        aih_categoryData();
        $color  = sanitize_hex_color($promptBaseData['color']);
        $responseData .= '<div class="box-media-prompt">';
        $responseData .= "<div class='box-media-image' style=background-color:$color>";

        $dom = new \DOMDocument();
        $svgIcon = $iconsDataArray[$promptBaseData['icon']];
        $dom->loadXML($svgIcon);
        $node = $dom->getElementsByTagName('svg')[0];
        $node->setAttribute('fill', "#fff");
        $iconImage = $dom->saveXML();
        $responseData .= $iconImage;
        $responseData .= '</div>';
        $responseData .= '<div class="box-media-content">';
        $responseData .= '<p class="title">';
        $responseData .= esc_html($promptBaseData['title']); // Escape HTML content.
        $responseData .= '</p>';
        $responseData .= '<p class="description">';
        $responseData .= esc_html($promptBaseData['description']); // Escape HTML content.
        $responseData .= '</p>';
        $responseData .= '</div>';
        $responseData .= '</div>';
        $responseData .= '</a>';
    }
    $responseData .= '</div>';
    $responseData .= '</div>';
    echo $responseData;
    // echo esc_attr($responseData);
    exit;
}

add_action('wp_ajax_aih_prompt_content_check_category', 'aih_prompt_content_check_category');
function aih_prompt_content_check_category()
{
    include(PLUGIN_DIR . 'includes/wp-global.php');
    $categoryValuesArray = sanitize_text_field($_POST['categoryValue']);
    $responseData = '';
    $promptBaseIncrement = 1;
    $responseData .= '<div class="panel-right-side-prompt">';
    $responseData .= '<input type="text" class="text-generator-search" id="search-value"><button id="search-prompt">Search</button>';
    $responseData .= '<div class="panel-right-side-prompt-container">';
    foreach ($promptBaseArray as $key => $promptBaseData) {

        $categortString = $promptBaseData['category'];

        $categoryArray = explode(',', $categortString);
        if (array_intersect($categoryArray, $categoryValuesArray)) {
            $responseData .= '<div class="box-media-prompt">';
            $responseData .= '<div class="box-media-image">';
            $responseData .= $iconsDataArray[$promptBaseData['icon']];
            $responseData .= '</div>';
            $responseData .= '<div class="box-media-content">';
            $responseData .= '<p class="title">';
            $responseData .= $promptBaseData['title'];
            $responseData .= '</p>';
            $responseData .= '<p class="description">';
            $responseData .= $promptBaseData['description'];
            $responseData .= $promptBaseData['category'];
            $responseData .= '</p>';
            $responseData .= '</div>';
            $responseData .= '</div>';
            // echo $promptBaseIncrement;
        }
    }
    $responseData .= '</div>';
    $responseData .= '</div>';
    echo $responseData;
    exit;
}

function aih_categoryData()
{
    include(PLUGIN_DIR . 'includes/wp-global.php');
}


add_action('init', 'aih_remove_cornerstone_menus');
function aih_remove_cornerstone_menus()
{
    remove_action('init', 'ai');
}


function aih_check_subscription()
{
    global $wpdb, $current_user, $apiUrl;
    wp_get_current_user();
    $user_email = $current_user->user_email;

    // Set up the API request
    $login_api_url =  $apiUrl . 'login';

    $login_api_request_args = array(
        'headers' => array(
            'Accept' => 'application/json',
            'userType' => 'shopify',
            'X-CSRF-TOKEN' => '',
        ),
        'body' => array(
            'email' => $user_email
        )
    );

    // Send the API request
    $response = wp_remote_post($login_api_url, $login_api_request_args);

    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body, true);

    $secret_key1 = $data['data']['secert_key'];

    // Set up the API request
    $api_url =  $apiUrl . 'check-subscription';

    $request_args = array(
        'headers' => array(
            'Accept' => 'application/json',
            'secertkey' => $secret_key1,
            'userType' => 'shopify',
            'X-CSRF-TOKEN' => '',
        ),
    );

    // Send the API request
    $response = wp_remote_post($api_url, $request_args);

    // Check if the request was successful
    if (is_wp_error($response)) {
        // Error handling if the API request fails
        return false; // Subscription check failed
    }

    // Parse the API response
    $body = wp_remote_retrieve_body($response);
    $data = json_decode($body, true);

    // Check the subscription status in the API response
    if ($data && isset($data['message']['status'])) {
        return $data['message']['status']; // Return the status value
    }

    return false; // Subscription status not found
}



// Creating Table After plugin install for Bulk Editor Savigng Post status
register_activation_hook(__FILE__, 'aih_api_response_logger_install');

function aih_api_response_logger_install()
{
    global $wpdb;
    $charset_collate = $wpdb->get_charset_collate();

    // Create the api_response_logs table
    $table_name_response_logs = $wpdb->prefix . 'api_response_logs';
    $sql_response_logs = "CREATE TABLE $table_name_response_logs (
        id INT(11) NOT NULL AUTO_INCREMENT,
        user_id VARCHAR(255) NOT NULL,
        title VARCHAR(255) NOT NULL,
        status VARCHAR(10) NOT NULL,
        PRIMARY KEY (id)
    ) $charset_collate;";
    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql_response_logs);

    // Create the settings table
    $table_name_settings = $wpdb->prefix . 'settings';
    $sql_settings = "CREATE TABLE $table_name_settings (
        id INT(11) NOT NULL AUTO_INCREMENT,
        user_id BIGINT(20) NOT NULL,
        time INT NOT NULL,
        attempt INT NOT NULL,
        PRIMARY KEY (id)
    ) $charset_collate;";
    dbDelta($sql_settings);
}

// Bulk Editor Post Save
add_action('wp_ajax_aih_publish_post_content', 'aih_publish_post_content');
add_action('wp_ajax_nopriv_aih_publish_post_content', 'aih_publish_post_content');

function aih_publish_post_content()
{
    // Retrieve the post data from the AJAX request
    $post_title = sanitize_text_field($_POST['title']);
    $post_content = wp_kses_post($_POST['content']);
    $post_category = sanitize_text_field($_POST['category']);
    $post_author_id = sanitize_text_field($_POST['admin']);
    $scheduled_time = sanitize_text_field($_POST['scheduled']); // Assuming date format from frontend

    // Retrieve WordPress timezone offset in seconds
    $wp_timezone_offset = get_option('gmt_offset') * 3600;

    // Convert scheduled time to Unix timestamp adjusted for WordPress timezone offset
    $scheduled_timestamp = strtotime($scheduled_time) - $wp_timezone_offset;

    // Format the scheduled time in UTC
    $formatted_scheduled_time = gmdate('Y-m-d H:i:s', $scheduled_timestamp);

    // Prepare the new post data
    $new_post = array(
        'post_title' => $post_title,
        'post_content' => $post_content,
        'post_date_gmt' => $formatted_scheduled_time,
        'post_status' => 'future', // Ensure post status is set to 'future' for scheduling
        'post_author' => $post_author_id,
        'post_category' => array($post_category),
    );

    // Insert the post into the database
    $post_id = wp_insert_post($new_post);

    if (is_wp_error($post_id)) {
        // Handle WP_Error if insertion fails
        wp_send_json_error('Failed to save or schedule the post. Error: ' . $post_id->get_error_message());
    } elseif ($post_id) {
        // Optionally add meta or perform other actions upon successful post insertion
        add_post_meta($post_id, 'BulkEditor', $post_id);

        // Optionally set categories
        if ($post_category) {
            wp_set_post_categories($post_id, array($post_category));
        }

        // Send the success response
        wp_send_json_success('Post successfully saved and scheduled for publishing.');
    } else {
        // Send the generic error response
        wp_send_json_error('Failed to save or schedule the post. Please try again.');
    }
}


// Save Bulk Editor API response data to the database
add_action('wp_ajax_aih_api_response', 'aih_api_response');
add_action('wp_ajax_nopriv_aih_api_response', 'aih_api_response');

function aih_api_response()
{
    global $wpdb;
    $table_name = $wpdb->prefix . 'api_response_logs';

    if (isset($_POST['title']) && isset($_POST['status'])) {
        $title = sanitize_text_field($_POST['title']);
        $status = sanitize_text_field($_POST['status']);
        $user_id = get_current_user_id();

        $wpdb->insert(
            $table_name,
            array(
                'title' => $title,
                'status' => $status,
                'user_id' => $user_id,
            ),
            array('%s', '%s', '%s')
        );
    }

    wp_die();
}

// Setting Page update
add_action('wp_ajax_aih_options_callback', 'aih_options_callback');
add_action('wp_ajax_nopriv_aih_options_callback', 'aih_options_callback');

function aih_options_callback()
{
    global $wpdb;

    if (isset($_POST['data']['time']) && isset($_POST['data']['attempt'])) {
        $time = sanitize_text_field($_POST['data']['time']);
        $attempt = sanitize_text_field($_POST['data']['attempt']);

        $user_id = get_current_user_id();

        // Save the options to the database
        $table_name_settings = $wpdb->prefix . 'settings';

        // Check if data exists for the user
        $existing_data = $wpdb->get_row(
            $wpdb->prepare(
                "SELECT * FROM $table_name_settings WHERE user_id = %d",
                $user_id
            ),
            ARRAY_A
        );

        if ($existing_data) {
            // User data exists, so update the existing data
            $wpdb->update(
                $table_name_settings,
                array(
                    'time' => $time,
                    'attempt' => $attempt,
                ),
                array('user_id' => $user_id),
                array('%d', '%d'),
                array('%d')
            );
        } else {
            // User data doesn't exist, so insert new data
            $wpdb->insert(
                $table_name_settings,
                array(
                    'user_id' => $user_id,
                    'time' => $time,
                    'attempt' => $attempt,
                ),
                array('%d', '%d', '%d')
            );
        }

        // Send success response back to the JavaScript
        wp_send_json_success('Settings saved successfully!');
    } else {
        // Send error response back to the JavaScript
        wp_send_json_error('Invalid data.');
    }
}

// Hook into WordPress Cron Job.
add_action('init', 'aih_schedule_custom_cron_job');

function aih_schedule_custom_cron_job()
{
    if (!wp_next_scheduled('ai_cron_event')) {
        $time_interval = aih_user_defined_time_interval();

        // Schedule the custom cron job based on $time_interval value.
        if ($time_interval == 60) {
            // Schedule the custom cron job to run hourly.
            wp_schedule_event(time(), 'hourly', 'ai_cron_event');
        } else {
            // Schedule the custom cron job to run every minute (default).
            wp_schedule_event(time(), 'Time', 'ai_cron_event');
        }
    }
}


add_filter('cron_schedules', 'ai_cron_interval');
function ai_cron_interval($schedules)
{
    $time_interval = aih_user_defined_time_interval();

    $attempt = aih_user_defined_attempt_interval();


    if ($attempt == 1) {

        if ($time_interval == 60) {

            $schedules['Time'] = array(
                'interval' => 60 * $time_interval,
                'display'  => 'Once Hourly',
            );

            return $schedules;
        } else {

            $schedules['Time'] = array(
                'interval' => 60 * $time_interval,
                'display'  => 'every ' . $time_interval . ' minutes',
            );

            return $schedules;
        }
    } else if ($attempt == 2) {

        if ($time_interval == 60) {

            $schedules['Time'] = array(
                'interval' => 60 * $time_interval,
                'display'  => 'Twice Hourly',
            );

            return $schedules;
        } else {

            $schedules['Time'] = array(
                'interval' => 60 * $time_interval,
                'display'  => 'every ' . $time_interval . ' minutes',
            );

            return $schedules;
        }
    } else {
        $schedules['Time'] = array(
            'interval' => 60 * $time_interval,
            'display'  => 'every ' . $time_interval . ' minutes',
        );

        return $schedules;
    }
}


// Function to get user-defined time interval from settings table.
function aih_user_defined_time_interval()
{
    global $wpdb;
    $user_id = get_current_user_id();
    $settings_table_name = $wpdb->prefix . 'settings'; // Replace with your settings table name.

    $settings_data = $wpdb->get_row($wpdb->prepare("SELECT time FROM $settings_table_name WHERE user_id = %d", $user_id));

    if ($settings_data) {
        $time_in_minutes = intval($settings_data->time);
        if ($time_in_minutes > 0) {
            return $time_in_minutes;
        }
    }

    return false;
}

// Get user-defined apttempts 
function aih_user_defined_attempt_interval()
{
    global $wpdb;
    $user_id = get_current_user_id();
    $settings_table_name = $wpdb->prefix . 'settings'; // Replace with your settings table name.

    $settings_data = $wpdb->get_row($wpdb->prepare("SELECT attempt FROM $settings_table_name WHERE user_id = %d", $user_id));

    if ($settings_data) {
        $attempt = intval($settings_data->attempt);
        if ($attempt > 0) {
            return $attempt;
        }
    }

    return false;
}


// Custom cron job function to process API requests and publish posts.
add_action('ai_cron_event', 'aih_process_cron_job');

function aih_process_cron_job()
{
    global $wpdb;
    $api_response_table_name = $wpdb->prefix . 'api_response_logs'; // Replace with your API response table name.
    $settings_table_name = $wpdb->prefix . 'settings'; // Replace with your settings table name.

    // Retrieve all titles and corresponding user IDs from the database where status is 'false'.
    $results = $wpdb->get_results("SELECT user_id, title FROM $api_response_table_name WHERE status = 'false'");

    foreach ($results as $result) {
        $user_id = $result->user_id;
        $title = $result->title;

        // Get the time and attempt data for the user from the settings table.
        $settings_data = $wpdb->get_row($wpdb->prepare("SELECT time, attempt FROM $settings_table_name WHERE user_id = %d", $user_id));

        if ($settings_data) {
            $time = intval($settings_data->time);
            $attempt = intval($settings_data->attempt);

            // Check if the user is allowed to make another API call based on the time and attempt.
            if ($attempt < 2 && $time > 0) {
                $api_data = aih_setting_api_function($title); // Pass the title to the API function.

                // Create a new WordPress post and publish the content.
                if ($api_data) {
                    $post_data = array(
                        'post_title'   => $title,
                        'post_content' => $api_data, // API-generated content.
                        'post_status'  => 'publish',
                        'post_author'  => $user_id,
                        'post_type'    => 'post',
                    );

                    // Insert the post into the database.
                    $post_id = wp_insert_post($post_data);

                    if ($post_id) {
                        // Increment the attempt count in the settings table.
                        $attempt++;
                        $wpdb->update($settings_table_name, array('attempt' => $attempt), array('user_id' => $user_id));

                        // Update the status in the API response table.
                        $wpdb->update($api_response_table_name, array('status' => 'true'), array('user_id' => $user_id, 'title' => $title));
                    } else {
                        // Log any error messages for failed post creations.
                        $error_messages[] = 'Error creating post for title: ' . $title;
                    }
                } else {
                    // Log any error messages for failed content generation.
                    $error_messages[] = 'Content generation failed for title: ' . $title;
                }
            }
        }
    }
}

// Code for saving cron job data
function aih_setting_api_function($title)
{
    global $apiUrl;
    // Make sure to set up the required variables like $secretKey, $openAI, and $title.
    $openAI = sanitize_text_field($_POST['openAI']);
    $secretKey = sanitize_text_field($_POST['secretKey']);

    $settingApi =  $apiUrl . 'get-setting';

    $settingApiData = array(
        'headers' => array(
            'Accept' => 'application/json',
            'secertkey' => $secretKey,
            'userType' => 'shopify',
            'X-CSRF-TOKEN' => '',
        ),
    );

    // Send the API request to generate content
    $setting_response = wp_remote_post($settingApi, $settingApiData);

    // Parse the API response to get content data
    $setting_body = wp_remote_retrieve_body($setting_response);
    $setting_data = json_decode($setting_body, true);

    $content_api_url =  $apiUrl . 'content-text';

    $content_request_args = array(
        'headers' => array(
            'Accept' => 'application/json',
            'openai' => $openAI,
            'secertkey' => $secretKey,
            'userType' => 'shopify',
            'X-CSRF-TOKEN' => '',
        ),
        'body' => array(
            'prompt' => 'Describe 2000 words in a paragraph ' . $title,
            'model' => 'gpt-3.5-turbo-instruct',
            'maxtoken' => 1500,
            'temperature' => 1,
        ),
    );

    // Send the API request to generate content
    $content_response = wp_remote_post($content_api_url, $content_request_args);

    // Parse the API response to get content data
    $content_body = wp_remote_retrieve_body($content_response);
    $content_data = json_decode($content_body, true);

    if (isset($content_data['data']['choices'][0]['text'])) {

        return $content_data['data']['choices'][0]['text'];
    } else {
        // Return an empty string or handle error
        $error_messages[] = 'Error creating post for title: ';
    }
}


// Rss Link
// Function to retrieve and process RSS feed data
function aih_retrieve_rss_data($rss_link)
{
    // Include the SimplePie library
    require_once(ABSPATH . WPINC . '/feed.php');

    // Create a new SimplePie feed instance
    $feed = fetch_feed($rss_link);

    // Check if the feed retrieval was successful
    if (!is_wp_error($feed)) {
        // Set the number of items to retrieve (optional)
        $max_items = $feed->get_item_quantity(10);

        // Get the feed items
        $items = $feed->get_items(0, $max_items);

        // Array to store post data
        $post_data = array();

        // Process the feed items
        foreach ($items as $item) {
            // Get the item title
            $title = $item->get_title();

            // Get the item description/content
            $description = $item->get_description();

            // Add post data to the array
            $post_data[] = array(
                'post_title'   => $title,
                'post_content' => $description,
                'post_status'  => 'publish',
                'post_author'  => 1, // Set the author ID
                'post_category' => array(1) // Set the category ID(s)
            );
        }

        // Create new posts
        $created_post_ids = array();
        foreach ($post_data as $data) {
            $post_id = wp_insert_post($data);
            if ($post_id) {
                // Post created successfully
                add_post_meta($post_id, 'RSS', $post_id);
                $created_post_ids[] = $post_id;
            } else {
                // Error creating post
                wp_send_json_error('Error creating post');
            }
        }

        if (!empty($created_post_ids)) {
            // Send a success response with the created post IDs
            wp_send_json_success($created_post_ids);
        } else {
            // No posts created
            wp_send_json_error('No posts created');
        }
    } else {
        // Error retrieving the feed
        wp_send_json_error('Error retrieving RSS feed: ' . $feed->get_error_message());
    }
}

// AJAX handler for retrieving RSS feed data

add_action('wp_ajax_aih_rss_link_data_handler', 'aih_rss_link_data_handler');
add_action('wp_ajax_nopriv_aih_rss_link_data_handler', 'aih_rss_link_data_handler');
function aih_rss_link_data_handler()
{
    if (isset($_POST['link'])) {
        $rss_link = sanitize_text_field($_POST['link']);
        aih_retrieve_rss_data($rss_link);
    }
    wp_die();
}



add_action('wp_ajax_aih_copy_paste_data', 'aih_copy_paste_data');
add_action('wp_ajax_nopriv_aih_copy_paste_data', 'aih_copy_paste_data');

function aih_copy_paste_data()
{
    global $apiUrl;
    $sanitizedTitles =  $_POST['indexedTitles'];

    $indexedTitles = array();

    foreach ($sanitizedTitles as $title) {
        $sanitizedTitle = sanitize_text_field($title);
        $indexedTitles[] = $sanitizedTitle;
    }

    $openAI = sanitize_text_field($_POST['openAI']);
    $secretKey = sanitize_text_field($_POST['secretKey']);
    $selectedValue = sanitize_text_field($_POST['selectedValue']);

    // Set up the API request to check the subscription status
    $subscription_api_url =  $apiUrl . 'check-subscription';

    $subscription_request_args = array(
        'headers' => array(
            'Accept' => 'application/json',
            'secertkey' => $secretKey,
            'userType' => 'shopify',
            'X-CSRF-TOKEN' => '',
        ),
    );

    // Send the API request to check the subscription status
    $subscription_response = wp_remote_post($subscription_api_url, $subscription_request_args);

    // Parse the API response to get subscription data
    $subscription_body = wp_remote_retrieve_body($subscription_response);
    $subscription_data = json_decode($subscription_body, true);

    $subscription_exist = ($subscription_data['message']['message'] === 'Subscription Exist.');

    // Determine the number of rows to send based on the subscription status
    $rowToSend = $subscription_exist ? count($indexedTitles) : min(5, count($indexedTitles));

    $success_counter = 0; // To count successful post creations
    $error_messages = array(); // To store any error messages during the process
    $quota_exceeded = false; // To check if quota error occurred


    $settingApi = $apiUrl . 'get-setting';

    $settingApiData = array(
        'headers' => array(
            'Accept' => 'application/json',
            'secertkey' => $secretKey,
            'userType' => 'shopify',
            'X-CSRF-TOKEN' => '',
        ),
    );

    // Send the API request to generate content
    $setting_response = wp_remote_get($settingApi, $settingApiData);

    // Parse the API response to get content data
    $setting_body = wp_remote_retrieve_body($setting_response);
    $setting_data = json_decode($setting_body, true);


    for ($i = 0; $i < $rowToSend; $i++) {
        $title = $indexedTitles[$i];

        // Set up the API request to generate content
        $content_api_url =  $apiUrl . 'content-text';

        $content_request_args = array(
            'headers' => array(
                'Accept' => 'application/json',
                'openai' => $openAI,
                'secertkey' => $secretKey,
                'userType' => 'shopify',
                'X-CSRF-TOKEN' => '',
            ),
            'body' => array(
                'prompt' => 'Describe 2000 words in a paragraph ' . $title,
                'model' => $setting_data['data']['model'],
                'maxtoken' => $setting_data['data']['maxtoken'],
                'temperature' => $setting_data['data']['temperature'],
            ),
            'timeout' => 30,
        );

        // Send the API request to generate content
        $content_response = wp_remote_post($content_api_url, $content_request_args);

        // Parse the API response to get content data
        $content_body = wp_remote_retrieve_body($content_response);
        $content_data = json_decode($content_body, true);

        if ($content_data && isset($content_data['message']) && $content_data['message'] === 'You exceeded your current quota, please check your plan and billing details.') {
            $quota_exceeded = true;
            break; // Exit the loop if quota exceeded
        }

        if (isset($content_data['data']['choices'][0]['text'])) {
            // Create a new blog post
            $post_data = array(
                'post_title'   => $title,
                'post_content' => $content_data['data']['choices'][0]['text'],
                'post_status'  => $selectedValue,
                'post_author'  => get_current_user_id(),
                'post_type'    => 'post',
            );

            // Insert the post into the database
            $post_id = wp_insert_post($post_data);

            // Increment the counter only for successful post creations
            if ($post_id) {
                $success_counter++;
            } else {
                // Log any error messages for failed post creations
                $error_messages[] = 'Error creating post for title: ' . $title;
            }
        } else {
            // Log any error messages for failed content generation
            $error_messages[] = 'Content generation failed for title: ' . $title . "\n";
        }

        // Add a 2-second delay before processing the next title
        sleep(2);
    }

    if ($quota_exceeded) {
        // Send the quota exceeded response
        wp_send_json_error('Quota exceeded. Please check your plan and billing details.');
    } elseif ($success_counter > 0) {
        // Send the success response
        wp_send_json_success('All posts generated and ' . $selectedValue . '.');
    } else {
        // Send the error response with error messages
        $error_message = implode("\n", $error_messages);
        wp_send_json_error($error_message);
    }

    // Send response back to the client-side script?
    die();
}


// Custom Mode Data
add_action('wp_ajax_aih_content_mode_data', 'aih_content_mode_data');
function aih_content_mode_data()
{
    $post_title = isset($_POST['title']) ? sanitize_text_field($_POST['title']) : '';
    $post_content = isset($_POST['content_text']) ? wp_kses_post($_POST['content_text']) : '';
    $post_excerpt = isset($_POST['excerpt_text']) ? sanitize_text_field($_POST['excerpt_text']) : '';

    $new = array(
        'post_title'   => $post_title,
        'post_content' => $post_content,
        'post_excerpt' => $post_excerpt,
        'post_status'  => 'publish' // Set the initial post status as 'publish'
    );

    $post_id = wp_insert_post($new);

    if ($post_id) {
        // Send the success response
        wp_send_json_success('Post successfully saved and published.');
    } else {
        // Send the error response
        wp_send_json_error('Something went wrong, try again.');
    }

    // Make sure to exit after sending the JSON response
    exit;
}

// Add the filter to modify the maximum file upload size
add_filter('aih_upload_size_limit', 'aih_upload_size_limit', 20);

function aih_upload_size_limit($size)
{
    // Set the maximum file size in bytes (e.g., 20 MB)
    $max_size = 20 * 1024 * 1024; // 20 MB in bytes

    // Return the new maximum size
    return $max_size;
}


// Post Delete In Queue
add_action('wp_ajax_aih_delete_post_action', 'aih_delete_post_action');
add_action('wp_ajax_nopriv_aih_delete_post_action', 'aih_delete_post_action'); // For non-logged-in users

function aih_delete_post_action()
{

    $post_id = intval($_POST['post_id']);

    // Perform the permanent delete action using the post ID.
    if (wp_delete_post($post_id, true)) {
        wp_send_json_success('Post deleted permanently!');
    } else {
        wp_send_json_error('Error deleting the post.');
    }
}

// Register the custom action hook to handle the AJAX request
add_action('wp_ajax_aih_google_sheet_data', 'aih_google_sheet_data');
add_action('wp_ajax_nopriv_aih_google_sheet_data', 'aih_google_sheet_data');

function aih_google_sheet_data()
{
    global $apiUrl;

    $sanitizedData = isset($_POST['body']) ? (array) $_POST['body'] : array();
    if (empty($sanitizedData)) {
        error_log("No data received from Google Sheet.");
        echo "Error: No data received.";
        return;
    }
    error_log("Received data from Google Sheet: " . print_r($sanitizedData, true));
    $jsonData = array();
    foreach ($sanitizedData as $innerArray) {
        $sanitizedInnerArray = array();
        foreach ($innerArray as $value) {
            $sanitizedValue = sanitize_text_field($value);
            $sanitizedInnerArray[] = $sanitizedValue;
        }
        $jsonData[] = $sanitizedInnerArray;
    }

    $openAI = isset($_POST['openAI']) ? sanitize_text_field($_POST['openAI']) : '';
    $secretKey = isset($_POST['secretKey']) ? sanitize_text_field($_POST['secretKey']) : '';
    $selectedDatetime = isset($_POST['selectedDatetime']) ? sanitize_text_field($_POST['selectedDatetime']) : '';

    $settingApi = $apiUrl . 'get-setting';
    $settingApiData = array(
        'headers' => array(
            'Accept' => 'application/json',
            'secertkey' => $secretKey,
            'userType' => 'shopify',
            'X-CSRF-TOKEN' => '',
        ),
    );

    $attempts = 3;
    $attempt = 0;

    do {
        $setting_response = wp_remote_get($settingApi, array(
            'headers' => $settingApiData['headers'],
            'timeout' => 120,
        ));

        if (is_wp_error($setting_response)) {
            $attempt++;
            sleep(5);
        } else {
            break;
        }
    } while ($attempt < $attempts);
    if (is_wp_error($setting_response)) {
        $error_message = $setting_response->get_error_message();
        error_log("Setting API Request Error: $error_message");
        echo "Error: " . esc_html($error_message);
        return;
    }

    $setting_body = wp_remote_retrieve_body($setting_response);
    $setting_data = json_decode($setting_body, true);

    if (empty($setting_data) || !isset($setting_data['data'])) {
        error_log("Error: Invalid setting data.");
        echo "Error: Invalid setting data.";
        return;
    }

    error_log("Setting data retrieved successfully.");
    $wp_timezone_offset = get_option('gmt_offset') * 3600;

    foreach ($jsonData as $row) {
        foreach ($row as $value) {
            $lineContent = sanitize_text_field($value);

            $apiEndpoint = $apiUrl . 'content-text';

            $request_data = array(
                'prompt' => 'Please generate a well-researched article on the topic of ' . $lineContent,
                'model' => $setting_data['data']['model'],
                'maxtoken' => $setting_data['data']['maxtoken'],
                'temperature' => $setting_data['data']['temperature'],
            );

            $headers = array(
                'accept' => 'application/json',
                'openai' => $openAI,
                'secertkey' => $secretKey,
                'userType' => 'shopify',
                'Content-Type' => 'application/json',
            );

            $args = array(
                'body' => wp_json_encode($request_data), // Convert the request data to JSON
                'headers' => $headers,
                'timeout' => 60,
            );

            $response = wp_remote_post($apiEndpoint, $args);

            if (is_wp_error($response)) {
                $error_message = $response->get_error_message();
                error_log("API Request Error: $error_message");
                continue;
            }

            $response_body = wp_remote_retrieve_body($response);
            $responseData = json_decode($response_body);

            if (isset($responseData->data->choices[0]->text)) {
                $content_data = $responseData->data->choices[0]->text;
            } else {
                error_log("Error: Invalid API response format.");
                continue;
            }

            $scheduled_timestamp = strtotime($selectedDatetime) - $wp_timezone_offset;

            if ($scheduled_timestamp === false) {
                error_log("Error: Invalid scheduled time format.");
                continue;
            }

            $formatted_scheduled_time = gmdate('Y-m-d H:i:s', $scheduled_timestamp);

            $post_data = array(
                'post_title'   => $lineContent,
                'post_content' => $content_data,
                'post_type'    => 'post',
                'post_date_gmt' => $formatted_scheduled_time,
                'post_status' => 'future',
            );

            $post_id = wp_insert_post($post_data, true); // Insert the post and get the post ID

            if (!is_wp_error($post_id)) {
                add_post_meta($post_id, 'GoogleSheet', $post_id);
                error_log('Post with ID ' . $post_id . ' generated and saved successfully.');
            } else {
                error_log('Error saving post: ' . $post_id->get_error_message());
            }
        }
    }

    echo 'Post generation process completed.';
}

// Google Sheet Api Key Auto Save
add_action('wp_ajax_save_google_key', 'save_google_key');

function save_google_key()
{
    if (is_user_logged_in()) {
        $user_id = get_current_user_id();

        $google_key = sanitize_text_field($_POST['google_key']);

        update_user_meta($user_id, 'googleKey', $google_key);

        wp_send_json_success('Google Key saved successfully!');
    } else {
        wp_send_json_error('User not logged in!');
    }

    wp_die();
}
// Fetch Google API Key
add_action('wp_ajax_get_google_key', 'get_google_key_callback');

function get_google_key_callback()
{
    if (is_user_logged_in()) {
        $user_id = get_current_user_id();

        $google_key = get_user_meta($user_id, 'googleKey', true);

        wp_send_json_success(array('googleKey' => $google_key));
    } else {
        wp_send_json_error('User not logged in!');
    }

    wp_die();
}



// Save CSV Post 
add_action('wp_ajax_aih_create_csv_wordpress_post', 'aih_create_csv_wordpress_post');
add_action('wp_ajax_nopriv_aih_create_csv_wordpress_post', 'aih_create_csv_wordpress_post'); // For non-logged-in users
function aih_create_csv_wordpress_post()
{
    if (isset($_POST['post_data'])) {
        $post_data = $_POST['post_data'];

        // Validate and sanitize the data
        $title = sanitize_text_field($post_data['title']);
        $content = sanitize_textarea_field($post_data['content']);
        $status = sanitize_text_field($post_data['status']);

        // Create the post using wp_insert_post()
        $new_post = array(
            'post_title'   => $title,
            'post_content' => $content,
            'post_status'  => $status,
            'post_type'    => 'post',
        );

        $post_id = wp_insert_post($new_post);

        if ($post_id) {
            wp_send_json_success(['post_id' => $post_id]);
        } else {
            wp_send_json_error('Error creating post.');
        }
    } else {
        wp_send_json_error('Post data not set.');
    }

    wp_die();
}


// Create table when plugin install for custom mode template save
register_activation_hook(__FILE__, 'aih_aiharness_install');

function aih_aiharness_install()
{
    global $wpdb;
    $charset_collate = $wpdb->get_charset_collate();

    // Create the custom_mode_template table
    $table_name_custom_mode_template = $wpdb->prefix . 'custom_mode_template';
    $sql_custom_mode_template = "CREATE TABLE $table_name_custom_mode_template (
        id INT(11) NOT NULL AUTO_INCREMENT,
        user_id BIGINT(20) NOT NULL,
        templateName VARCHAR(255) NOT NULL,
        postType VARCHAR(100) NOT NULL,
        Model VARCHAR(100) NOT NULL,
        Temperature FLOAT NOT NULL,
        Tokens INT(11) NOT NULL,
        topP FLOAT NOT NULL,
        bestOf INT(11) NOT NULL,
        Frequency VARCHAR(100) NOT NULL,
        Presence VARCHAR(100) NOT NULL,
        PromptTitle TEXT NOT NULL,
        PromptSection TEXT NOT NULL,
        PromptContent TEXT NOT NULL,
        PromptExcerpt TEXT NOT NULL,
        PromptMeta TEXT NOT NULL,
        PRIMARY KEY (id)
    ) $charset_collate;";
    dbDelta($sql_custom_mode_template);
}

// save custom mode template data in database table
add_action('wp_ajax_aih_template_data', 'aih_template_data');
add_action('wp_ajax_nopriv_aih_template_data', 'aih_template_data');

function aih_template_data()
{
    global $wpdb;
    global $current_user;
    wp_get_current_user();
    $user_id = $current_user->ID;

    $templateName = isset($_POST['templateName']) ? sanitize_text_field($_POST['templateName']) : '';
    $postType = isset($_POST['postType']) ? sanitize_text_field($_POST['postType']) : '';
    $model = isset($_POST['model']) ? sanitize_text_field($_POST['model']) : '';
    $temperature = isset($_POST['temperature']) ? absint($_POST['temperature']) : 0;
    $maxTokens = isset($_POST['maxTokens']) ? absint($_POST['maxTokens']) : 0;
    $topP = isset($_POST['topP']) ? absint($_POST['topP']) : 0;
    $bestOf = isset($_POST['bestOf']) ? absint($_POST['bestOf']) : 0;
    $frequency = isset($_POST['frequency']) ? sanitize_text_field($_POST['frequency']) : '';
    $presence = isset($_POST['presence']) ? sanitize_text_field($_POST['presence']) : '';
    $promptTitle = isset($_POST['promptTitle']) ? sanitize_text_field($_POST['promptTitle']) : '';
    $promptSection = isset($_POST['promptSection']) ? sanitize_text_field($_POST['promptSection']) : '';
    $promptContent = isset($_POST['promptContent']) ? sanitize_text_field($_POST['promptContent']) : '';
    $promptExcerpt = isset($_POST['promptExcerpt']) ? sanitize_text_field($_POST['promptExcerpt']) : '';
    $promptMeta = isset($_POST['promptMeta']) ? sanitize_text_field($_POST['promptMeta']) : '';


    // Perform validation and sanitation of the data if needed

    // Check if the template name exists in the database
    $table_name = $wpdb->prefix . 'custom_mode_template';
    $existing_template = $wpdb->get_row(
        $wpdb->prepare("SELECT * FROM $table_name WHERE user_id = %d AND templateName = %s", $user_id, $templateName)
    );

    if ($existing_template) {
        // If the template exists, update the existing data
        $wpdb->update(
            $table_name,
            array(
                'postType' => $postType,
                'Model' => $model,
                'Temperature' => $temperature,
                'Tokens' => $maxTokens,
                'topP' => $topP,
                'bestOf' => $bestOf,
                'Frequency' => $frequency,
                'Presence' => $presence,
                'PromptTitle' => $promptTitle,
                'PromptSection' => $promptSection,
                'PromptContent' => $promptContent,
                'PromptExcerpt' => $promptExcerpt,
                'PromptMeta' => $promptMeta,
            ),
            array('user_id' => $user_id, 'templateName' => $templateName)
        );
    } else {
        // If the template does not exist, insert new data
        $wpdb->insert(
            $table_name,
            array(
                'user_id' => $user_id,
                'templateName' => $templateName,
                'postType' => $postType,
                'Model' => $model,
                'Temperature' => $temperature,
                'Tokens' => $maxTokens,
                'topP' => $topP,
                'bestOf' => $bestOf,
                'Frequency' => $frequency,
                'Presence' => $presence,
                'PromptTitle' => $promptTitle,
                'PromptSection' => $promptSection,
                'PromptContent' => $promptContent,
                'PromptExcerpt' => $promptExcerpt,
                'PromptMeta' => $promptMeta,
            )
        );
    }

    // Send the response back to the client-side script
    wp_send_json_success('Template saved successfully!');
    die();
}

// Fetch Custom mode tempplate data

// Enqueue jQuery
function aih_enqueue_custom_scripts()
{
    wp_enqueue_script('jquery');
}
add_action('wp_enqueue_scripts', 'aih_enqueue_custom_scripts');

// AJAX handler for fetching data
function aih_fetch_data_based_on_template()
{
    global $wpdb;

    if (isset($_POST['template'])) {
        $selectedTemplate = sanitize_text_field($_POST['template']);
        $tableNameUser = $wpdb->prefix . 'custom_mode_template';
        $user_id = get_current_user_id();

        $data = $wpdb->get_results(
            $wpdb->prepare(
                "SELECT * FROM $tableNameUser WHERE templateName = %s AND user_id = %d",
                $selectedTemplate,
                $user_id
            )
        );

        $formatted_data = array();
        foreach ($data as $item) {
            $formatted_data[] = array(
                'templateName' => $item->templateName,
                'postType' => $item->postType,
                'Model' => $item->Model,
                'Temperature' => $item->Temperature,
                'Tokens' => $item->Tokens,
                'topP' => $item->topP,
                'bestOf' => $item->bestOf,
                'Frequency' => $item->Frequency,
                'Presence' => $item->Presence,
                'PromptTitle' => $item->PromptTitle,
                'PromptSection' => $item->PromptSection,
                'PromptContent' => $item->PromptContent,
                'PromptExcerpt' => $item->PromptExcerpt,
                'PromptMeta' => $item->PromptMeta,
            );
        }

        wp_send_json($formatted_data); // Send the formatted data as JSON
    }

    die(); // Always use die() at the end of AJAX functions
}


// Callback function to save API key
function save_api_key_callback()
{
    if (isset($_POST['api_key'])) {
        global $wpdb, $current_user, $user_id;
        wp_get_current_user();
        $user_id = $current_user->ID;
        $api_key = sanitize_text_field($_POST['api_key']);
        $api_user_id = sanitize_text_field($_POST['api_user_id']);

        $tablenameSetting = $wpdb->prefix . 'setting';
        $existing_api_key = $wpdb->get_results("SELECT * FROM `$tablenameSetting` where api_user_id=$api_user_id", ARRAY_A);

        if (count($existing_api_key) > 0) {
            $result = $wpdb->update(
                $tablenameSetting,
                array('api_key' => $api_key, 'user_id' => $user_id),
                array('api_user_id' => $api_user_id)
            );
        } else {
            $data_to_insert = array_merge(array('api_user_id' => 1), array('api_user_id' => $api_user_id));
            $data_to_insert['api_key'] = $api_key;
            $data_to_insert['user_id'] = $user_id;

            $result = $wpdb->insert(
                $tablenameSetting,
                $data_to_insert
            );
        }

        if ($result === false) {
            wp_send_json_error('Failed to save API key.');
        } else {
            wp_send_json_success('API key saved successfully.');
        }

        wp_die();
    } else {
        wp_send_json_error('API key not provided.');
        wp_die();
    }
}

// Hook the custom action
add_action('wp_ajax_save_api_key', 'save_api_key_callback');
function custom_plugin_start_session()
{
    if (!session_id()) {
        session_start();
    }
}
add_action('init', 'custom_plugin_start_session');


function store_user_data_in_session()
{
    isset($_POST['user_id']) ? $_SESSION['user_id'] = sanitize_text_field($_POST['user_id']) : '';
    isset($_POST['userEmail']) ? $_SESSION['userEmail'] = sanitize_text_field($_POST['userEmail']) : '';
    isset($_POST['userSub']) ? $_SESSION['userSub'] = sanitize_text_field($_POST['userSub']) : '';
    isset($_POST['secret_key']) ? $_SESSION['secret_key'] = sanitize_text_field($_POST['secret_key']) : '';
    isset($_POST['personal_key']) ? $_SESSION['personal_key'] = sanitize_text_field($_POST['personal_key']) : '';
}
add_action('wp_ajax_store_user_data', 'store_user_data_in_session');


// Enqueue the JavaScript file
function custom_plugin_enqueue_scripts()
{
    wp_enqueue_script('session-management', plugins_url('session-management.js', __FILE__), array('jquery'), '1.0', true);
}
add_action('wp_enqueue_scripts', 'custom_plugin_enqueue_scripts');


add_action('wp_ajax_clear_session', 'clear_session');
function clear_session()
{
    session_start();
    session_unset();
    session_destroy();
    wp_send_json_success('Session cleared successfully.');
}
